@@ -10,13 +10,13 @@ trap finish EXIT
 
 cd $(dirname ${BASH_SOURCE[0]})
 
-if ! source ./pyenv-3.3-3.5-versions ; then
+if ! source ./pyenv-3.1-3.2-versions ; then
     exit $?
 fi
 cd ..
 [[ -f test/.python-version ]] && rm -v test/.python-version
-(cd ../python-xdis/admin-tools && source ./setup-python-3.3.sh)
-git checkout python-3.3
+(cd ../python-xdis/admin-tools && source ./setup-python-3.1.sh)
+git checkout python-3.1-to-3.2
 
 for version in $PYVERSIONS; do
     echo --- $version ---
