@@ -3,24 +3,25 @@
 
 Note: this is subclassed. Later versions use operations from here.
 """
-from __future__ import print_function, division
 
+import logging
+import operator
+import sys
 from collections import namedtuple
 
-Version_info = namedtuple("version_info", "major minor micro releaselevel serial")
+from xdis.version_info import PYTHON_VERSION_TRIPLE
 
 # FIXME: we should use:
 # from copy import deepcopy
 
-import operator
-import logging
-import six
-import sys
-from xdis import PYTHON_VERSION
 
-if PYTHON_VERSION > 2.7:
+if PYTHON_VERSION_TRIPLE >= (3, 0):
     import importlib
 
+    import_fn = importlib.__import__
+else:
+    import_fn = __import__
+
 from xpython.byteop.byteop import (
     ByteOpBase,
     fmt_binary_op,
@@ -30,6 +31,8 @@ from xpython.byteop.byteop import (
 from xpython.pyobj import Cell, Function, traceback_from_frame
 from xpython.vmtrace import PyVMEVENT_RETURN, PyVMEVENT_YIELD
 
+Version_info = namedtuple("version_info", "major minor micro releaselevel serial")
+
 log = logging.getLogger(__name__)
 
 # Code with these names have an implicit .0 in them
@@ -86,6 +89,10 @@ def fmt_make_function(vm, arg=None, repr=repr):
 class ByteOp24(ByteOpBase):
     def __init__(self, vm):
         super(ByteOp24, self).__init__(vm)
+
+        # Superclasses can overwrite this. But default to not PyPy.
+        self.is_pypy = False
+
         self.stack_fmt["MAKE_FUNCTION"] = fmt_make_function
         # Not for 2.4
         self.stack_fmt["CALL_FUNCTION"] = fmt_call_function
@@ -94,7 +101,7 @@ class ByteOp24(ByteOpBase):
         ).split():
             self.stack_fmt[opname] = fmt_binary_op
 
-        for opname in ("ROT_THREE STORE_SUBSCR EXEC_STMT BUILD_CLASS").split():
+        for opname in "ROT_THREE STORE_SUBSCR EXEC_STMT BUILD_CLASS".split():
             self.stack_fmt[opname] = fmt_ternary_op
 
         for opname in (
@@ -119,7 +126,7 @@ class ByteOp24(ByteOpBase):
         return " (%s)" % vm.peek(1)
 
     def BRKPT(self):
-        """Psuedo opcode: breakpoint. We added this. TODO: call callback, then run
+        """Pseudo opcode: breakpoint. We added this. TODO: call callback, then run
         instruction that should have gotten run.
         """
         vm = self.vm
@@ -171,27 +178,29 @@ class ByteOp24(ByteOpBase):
     ############################################################################
 
     def NOP(self):
-        "Do nothing code. Used as a placeholder by the bytecode optimizer."
+        """Do nothing code. Used as a placeholder by the bytecode optimizer."""
         pass
 
     # Stack manipulation
 
     def POP_TOP(self):
-        "Removes the top-of-stack (TOS) item."
+        """Removes the top-of-stack (TOS) item."""
         self.vm.pop()
 
     def ROT_TWO(self):
-        "Swaps the two top-most stack items."
+        """Swaps the two top-most stack items."""
         a, b = self.vm.popn(2)
         self.vm.push(b, a)
 
     def ROT_THREE(self):
-        "Lifts second and third stack item one position up, moves top down to position three."
+        """Lifts second and third stack item one position up, moves
+        top down to position three."""
         a, b, c = self.vm.popn(3)
         self.vm.push(c, a, b)
 
     def ROT_FOUR(self):
-        "Lifts second, third and forth stack item one position up, moves top down to position four."
+        """Lifts second, third and forth stack item one position up,
+        moves top down to position four."""
         a, b, c, d = self.vm.popn(4)
         self.vm.push(d, a, b, c)
 
@@ -261,12 +270,12 @@ class ByteOp24(ByteOpBase):
         self.vm.return_value = dest
         return "continue"
 
-    def LIST_APPEND(self, count):
+    def LIST_APPEND(self):
         """Calls list.append(TOS1, TOS). Used to implement list
         comprehensions.
         """
         val = self.vm.pop()
-        the_list = self.vm.peek(count)
+        the_list = self.vm.pop()
         the_list.append(val)
 
     def LOAD_LOCALS(self):
@@ -314,7 +323,7 @@ class ByteOp24(ByteOpBase):
             globs = self.vm.frame.f_globals
         if locs is None:
             locs = self.vm.frame.f_locals
-        six.exec_(stmt, globs, locs)
+        exec(stmt, globs, locs)
 
     def POP_BLOCK(self):
         """
@@ -334,7 +343,7 @@ class ByteOp24(ByteOpBase):
             why = v
             if why in ("return", "continue"):
                 self.vm.return_value = self.vm.pop()
-            if why == "silenced":  # self.float_version >= 3.0
+            if why == "silenced":  # self.version_info[:2] >= (3, 0)
                 block = self.vm.pop_block()
                 assert block.type == "except-handler"
                 self.vm.unwind_block(block)
@@ -346,7 +355,7 @@ class ByteOp24(ByteOpBase):
             val = self.vm.pop()
             tb = self.vm.pop()
             self.vm.last_exception = (exctype, val, tb)
-            if self.float_version >= 3.5:
+            if self.version_info[:2] >= (3, 5):
                 block = self.vm.top_block()
                 while len(self.vm.frame.stack) > block.level:
                     self.vm.pop()
@@ -366,7 +375,7 @@ class ByteOp24(ByteOpBase):
         # Note: type() wants to only create new-style classes, while
         # bases might include only old-style classes. This will
         # trigger this error: TypeError: a new-style class can't have
-        # only classic bases So what we'll do is thow in "object" so
+        # only classic bases So what we'll do is throw in "object" so
         # there is at least one new-style class.
         try:
             klass = type(name, bases, methods)
@@ -380,8 +389,13 @@ class ByteOp24(ByteOpBase):
         STORE_GLOBAL if possible."""
         self.vm.frame.f_locals[name] = self.vm.pop()
 
+    def DELETE_GLOBAL(self, name):
+        """Implements del name, where name in global."""
+        del self.vm.frame.f_globals[name]
+
     def DELETE_NAME(self, name):
-        """Implements del name, where namei is the index into co_names attribute of the code object."""
+        """Implements del name, where name is the index into co_names
+        attribute of the code object."""
         del self.vm.frame.f_locals[name]
 
     def UNPACK_SEQUENCE(self, count):
@@ -412,7 +426,7 @@ class ByteOp24(ByteOpBase):
         delattr(obj, name)
 
     def STORE_GLOBAL(self, name):
-        "Works as STORE_NAME, but stores the name as a global."
+        """Works as STORE_NAME, but stores the name as a global."""
         f = self.vm.frame
         f.f_globals[name] = self.vm.pop()
 
@@ -433,7 +447,9 @@ class ByteOp24(ByteOpBase):
         # except NameError:
         #     self.vm.last_traceback = traceback_from_frame(self.vm.frame)
         #     tb  = traceback_from_frame(self.vm.frame)
-        #     self.vm.last_exception = (NameError, NameError("name '%s' is not defined" % name), tb)
+        #     self.vm.last_exception = (NameError,
+        #                               NameError("name '%s' is not defined" % name),
+        #                               tb)
         #     return "exception"
         # else:
         #     self.vm.push(self.lookup_name(name))
@@ -461,8 +477,8 @@ class ByteOp24(ByteOpBase):
         Pushes a new dictionary object onto the stack. The dictionary is
         pre-sized to hold count entries.
         """
-        # "size" is ignored; In contrast to C, in Python, the default dictionary type has no
-        # notion of allocation size.
+        # "size" is ignored; In contrast to C, in Python, the default
+        # dictionary type has no notion of allocation size.
         self.vm.push({})
 
     # end BUILD_ operators
@@ -494,7 +510,8 @@ class ByteOp24(ByteOpBase):
     ]
 
     def COMPARE_OP(self, opname):
-        """Performs a Boolean operation. The operation name can be found in cmp_op[opname]."""
+        """Performs a Boolean operation. The operation name can be
+        found in cmp_op[opname]."""
         x, y = self.vm.popn(2)
         self.vm.push(self.COMPARE_OPERATORS[opname](x, y))
 
@@ -511,23 +528,17 @@ class ByteOp24(ByteOpBase):
         """
         frame = self.vm.frame
 
-        if PYTHON_VERSION > 2.7:
-            module = importlib.__import__(
-                name, frame.f_globals, frame.f_locals, fromlist=None, level=0
-            )
-        else:
-            module = __import__(
-                name, frame.f_globals, frame.f_locals, fromlist=None, level=0
-            )
+        module = import_fn(
+            name, frame.f_globals, frame.f_locals, fromlist=None, level=0
+        )
 
         # FIXME: generalize this
         if name in sys.builtin_module_names:
             # FIXME: do more here.
-            if PYTHON_VERSION != self.float_version:
+            if PYTHON_VERSION_TRIPLE[:2] != self.version_info[:2]:
                 if name == "sys":
                     module.hexversion = self.hexversion
                     module.version_info = self.version_info
-                    module.version = self.float_version
                     pass
                 pass
         # FIXME:
@@ -544,17 +555,23 @@ class ByteOp24(ByteOpBase):
         """
         mod = self.vm.top()
         if not hasattr(mod, name):
-            value = ImportError(
-                "cannot import name '%s' from '%s' (%s)"
-                % (name, mod.__name__, mod.__file__)
-            )
+            if not hasattr(mod, "__file__"):
+                # Builtins don't have a __file__ attribute
+                value = ImportError(
+                    "cannot import name '%s' from '%s'" % (name, mod.__name__)
+                )
+            else:
+                value = ImportError(
+                    "cannot import name '%s' from '%s' (%s)"
+                    % (name, mod.__name__, mod.__file__)
+                )
 
             self.vm.last_exception = (ImportError, value, None)
             return "exception"
 
         self.vm.push(getattr(mod, name))
 
-    ## Jumps
+    # Jumps
 
     def JUMP_FORWARD(self, jump_offset):
         """Increments bytecode counter by jump.
@@ -654,14 +671,14 @@ class ByteOp24(ByteOpBase):
         self.vm.push_block("finally", jump_offset)
 
     def STORE_MAP(self):
-        """
-        Store a key and value pair in a dictionary. Pops the key and value while leaving the dictionary on the stack.
+        """Store a key and value pair in a dictionary. Pops the key
+        and value while leaving the dictionary on the stack.
         """
         the_map, val, key = self.vm.popn(3)
         the_map[key] = val
         self.vm.push(the_map)
 
-    ## some (but not all) Names
+    # some (but not all) Names
 
     def LOAD_FAST(self, name):
         """
@@ -684,10 +701,10 @@ class ByteOp24(ByteOpBase):
         del self.vm.frame.f_locals[var_num]
 
     def LOAD_CLOSURE(self, i):
-        """
-        Pushes a reference to the cell contained in slot i of the cell and
-        free variable storage. The name of the variable is co_cellvars[i] if i is less
-        than the length of co_cellvars. Otherwise it is co_freevars[i -len(co_cellvars)].
+        """Pushes a reference to the cell contained in slot i of the
+        cell and free variable storage. The name of the variable is
+        co_cellvars[i] if i is less than the length of
+        co_cellvars. Otherwise it is co_freevars[i -len(co_cellvars)].
         """
         self.vm.push(self.vm.frame.cells[i])
 
@@ -700,8 +717,8 @@ class ByteOp24(ByteOpBase):
         self.vm.push(self.vm.frame.cells[name].get())
 
     def STORE_DEREF(self, name):
-        """
-        Stores TOS into the cell contained in slot i of the cell and free variable storage.
+        """Stores TOS into the cell contained in slot i of the cell
+        and free variable storage.
         """
         self.vm.frame.cells[name].set(self.vm.pop())
 
@@ -713,7 +730,7 @@ class ByteOp24(ByteOpBase):
             """
             This opcode is obsolete.
 
-            It we last seen in Python 2.2.
+            It was last seen in Python 2.2.
             """
             self.vm.frame.f_lineno = lineno
 
@@ -790,6 +807,7 @@ class ByteOp24(ByteOpBase):
         # NOTE: the dis docs quoted above are completely wrong about the order of the
         # operands on the stack!
         tb = None
+        exctype = None
         if argc == 0:
             exctype, val, tb = self.vm.last_exception
         elif argc == 1:
@@ -798,15 +816,15 @@ class ByteOp24(ByteOpBase):
         elif argc == 2:
             val = self.vm.pop()
             # Investigate: right now we see this *only* in 2.6.
-            # Can it happen in other bytecode vesrions?
-            if self.float_version == 2.6:
+            # Can it happen in other bytecode versions?
+            if self.version_info[:2] == (2, 6):
                 val = AssertionError(val)
             exctype = self.vm.pop()
         elif argc == 3:
             tb = self.vm.pop()
             val = self.vm.pop()
             # See comment above
-            if self.float_version == 2.6:
+            if self.version_info[:2] == (2, 6):
                 val = AssertionError(val)
             exctype = self.vm.pop()
 
@@ -845,7 +863,7 @@ class ByteOp24(ByteOpBase):
             return self.call_function(argc, var_args=[], keyword_args={})
         except TypeError as exc:
             tb = self.vm.last_traceback = traceback_from_frame(self.vm.frame)
-            self.vm.last_exception = (TypeError, exc.args, tb)
+            self.vm.last_exception = (TypeError, exc, tb)
             return "exception"
 
     def CALL_FUNCTION_VAR(self, argc):
