@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
 """Byte Interpreter operations for Python 3.2
 """
-from __future__ import print_function, division
 
+import inspect
 
+from xdis.opcodes.opcode_3x import parse_fn_counts_30_35
 from xpython.byteop.byteop24 import ByteOp24, Version_info
 from xpython.byteop.byteop27 import ByteOp27
 from xpython.pyobj import Function
@@ -25,12 +26,12 @@ del ByteOp24.RAISE_VARARGS
 del ByteOp24.DUP_TOPX
 
 
-def fmt_make_function(vm, arg=None, repr=repr):
+def fmt_make_function(vm, arg=None, repr_fn=repr):
     """
     returns the name of the function from the code object in the stack
     """
     # Gotta love Python for stuff like this.
-    fn_index = 1 if vm.version >= 3.2 else 2
+    fn_index = 1 if vm.version >= (3, 2) else 2
     fn_item = vm.peek(fn_index)
     name = fn_item if isinstance(fn_item, str) else fn_item.co_name
     return " (%s)" % name
@@ -43,74 +44,8 @@ class ByteOp32(ByteOp27):
         self.version = "3.2.6 (default, Oct 27 1955, 00:00:00)\n[x-python]"
         self.version_info = Version_info(3, 2, 6, "final", 0)
 
-    # Changed from 2.7
-    # 3.2 has kwdefaults that aren't allowed in 2.7
-    def MAKE_FUNCTION(self, argc):
-        """
-        Pushes a new function object on the stack. From bottom to top, the consumed stack must consist of:
-
-        * argc & 0xFF default argument objects in positional order
-        * (argc >> 8) & 0xFF pairs of name and default argument, with the name just below the object on the stack, for keyword-only parameters
-        * (argc >> 16) & 0x7FFF parameter annotation objects
-        * a tuple listing the parameter names for the annotations (only if there are ony annotation objects)
-        * the code associated with the function (at TOS1 if 3.3+ else at TOS for 3.0..3.2)
-        * the qualified name of the function (at TOS if 3.3+)
-        """
-        rest, default_count = divmod(argc, 256)
-        annotate_count, kw_default_count = divmod(rest, 256)
-
-        if self.float_version >= 3.3:
-            name = self.vm.pop()
-            code = self.vm.pop()
-        else:
-            code = self.vm.pop()
-            name = code.co_name
-
-        if annotate_count:
-            annotate_names = self.vm.pop()
-            # annotate count includes +1 for the above names
-            annotate_objects = self.vm.popn(annotate_count - 1)
-            n = len(annotate_names)
-            assert n == len(annotate_objects)
-            annotations = {annotate_names[i]: annotate_objects[i] for i in range(n)}
-        else:
-            annotations = {}
-
-        if kw_default_count:
-            kw_default_pairs = self.vm.popn(2 * kw_default_count)
-            kwdefaults = dict(
-                kw_default_pairs[i : i + 2] for i in range(0, len(kw_default_pairs), 2)
-            )
-        else:
-            kwdefaults = {}
-
-        if default_count:
-            defaults = self.vm.popn(default_count)
-        else:
-            defaults = tuple()
-
-        # FIXME: DRY with code in byteop3{2,6}.py
-
-        globs = self.vm.frame.f_globals
-
-        fn = Function(
-            name=name,
-            code=code,
-            globs=globs,
-            argdefs=tuple(defaults),
-            closure=None,
-            vm=self.vm,
-            kwdefaults=kwdefaults,
-            annotations=annotations,
-        )
-
-        self.vm.push(fn)
-
-    # Order of function here is the same as in:
-    # https://docs.python.org/3.2/library/dis.html#python-bytecode-instructions
-
-    # Note these are only the functions that aren't in the parent class
-    # here, Python 2.7
+    def convert_native_to_Function(self, frame, func):
+        assert inspect.isfunction(func) or isinstance(func, Function)
 
     def DUP_TOP_TWO(self):
         """Duplicates the reference on top of the stack."""
@@ -132,15 +67,129 @@ class ByteOp32(ByteOp27):
         self.vm.unwind_block(block)
 
     def LOAD_BUILD_CLASS(self):
-        """Pushes builtins.__build_class__() onto the stack. It is later called by CALL_FUNCTION to construct a class."""
+        """Pushes builtins.__build_class__() onto the stack. It is
+        later called by CALL_FUNCTION to construct a class."""
         self.vm.push(__build_class__)
 
+    def MAKE_CLOSURE(self, argc: int):
+        """
+        Creates a new function object, sets its ``__closure__`` slot, and
+        pushes it on the stack. TOS is the code qualified name of the
+        function, TOS is the code associated with the function and
+        TOS1 is the tuple containing cells for the closure's free
+        variables. The function asl has ``argc`` default parameters,
+        which are found below the cells.
+        """
+        default_count, kw_default_count, annotate_count = parse_fn_counts_30_35(argc)
+        closure, code = self.vm.popn(2)
+
+        if kw_default_count:
+            kw_default_pairs = self.vm.popn(2 * kw_default_count)
+            kwdefaults = dict(
+                kw_default_pairs[i : i + 2] for i in range(0, len(kw_default_pairs), 2)
+            )
+        else:
+            kwdefaults = {}
+
+        if default_count:
+            defaults = self.vm.popn(default_count)
+        else:
+            defaults = tuple()
+
+        if annotate_count:
+            annotate_names = self.vm.pop()
+            annotate_types = self.vm.popn(annotate_count)
+            n = len(annotate_names)
+            assert n == len(annotate_types)
+            annotations = {annotate_names[i]: annotate_types[i] for i in range(n)}
+        else:
+            annotations = {}
+
+        # FIXME: DRY with code in MAKE_FUNCTION
+
+        globs = self.vm.frame.f_globals
+
+        fn = Function(
+            name=code.co_name,
+            code=code,
+            globs=globs,
+            argdefs=tuple(defaults),
+            closure=closure,
+            vm=self.vm,
+            kwdefaults=kwdefaults,
+            annotations=annotations,
+        )
+
+        self.vm.push(fn)
+
+    # Changed from 2.7
+    # 3.2 has kwdefaults that aren't allowed in 2.7
+    def MAKE_FUNCTION(self, argc: int):
+        """
+        Pushes a new function object on the stack. From bottom to top, the consumed
+        stack must consist of:
+
+        * argc & 0xFF default argument objects in positional order
+        * (argc >> 8) & 0xFF pairs of name and default argument, with the name just
+          below the object on the stack, for keyword-only parameters
+        * (argc >> 16) & 0x7FFF parameter annotation objects
+        * a tuple listing the parameter names for the annotations (only if there are
+          only annotation objects)
+        * the code associated with the function (at TOS1 if 3.3+ else at TOS for
+          3.0..3.2)
+        * the qualified name of the function (at TOS if 3.3+)
+        """
+        default_count, kw_default_count, annotate_count = parse_fn_counts_30_35(argc)
+
+        code = self.vm.pop()
+        name = code.co_name
+
+        if kw_default_count:
+            kw_default_pairs = self.vm.popn(2 * kw_default_count)
+            kwdefaults = dict(
+                kw_default_pairs[i : i + 2] for i in range(0, len(kw_default_pairs), 2)
+            )
+        else:
+            kwdefaults = {}
+
+        if default_count:
+            defaults = self.vm.popn(default_count)
+        else:
+            defaults = tuple()
+
+        if annotate_count:
+            annotate_names = self.vm.pop()
+            # annotate count includes +1 for the above names
+            annotate_objects = self.vm.popn(annotate_count - 1)
+            n = len(annotate_names)
+            assert n == len(annotate_objects)
+            annotations = {annotate_names[i]: annotate_objects[i] for i in range(n)}
+        else:
+            annotations = {}
+
+        # FIXME: DRY with code in byteop3{2,6}.py
+
+        globs = self.vm.frame.f_globals
+
+        fn = Function(
+            name=name,
+            code=code,
+            globs=globs,
+            argdefs=tuple(defaults),
+            closure=None,
+            vm=self.vm,
+            kwdefaults=kwdefaults,
+            annotations=annotations,
+        )
+
+        self.vm.push(fn)
+
     # This opcode disappears starting in 3.5
     def WITH_CLEANUP(self):
         """Cleans up the stack when a `with` statement block exits. TOS is the
         context manager's `__exit__()` bound method.
 
-        Below TOS are 1-3 values indicating how/why the finally clause
+        Below TOS are 1-3 values indicating how/why the "finally" clause
         was entered:
 
         * SECOND = None
@@ -189,10 +238,11 @@ class ByteOp32(ByteOp27):
 
     # Note: this is gone in 3.4
     def STORE_LOCALS(self):
-        """Pops TOS from the stack and stores it as the current frames f_locals. This is used in class construction."""
+        """Pops TOS from the stack and stores it as the current frames
+        f_locals. This is used in class construction."""
         self.vm.frame.f_locals = self.vm.pop()
 
-    def RAISE_VARARGS(self, argc):
+    def RAISE_VARARGS(self, argc: int):
         """
         Raises an exception. argc indicates the number of arguments to the
         raise statement, ranging from 0 to 3. The handler will find
