@@ -4,7 +4,9 @@
 import inspect
 import types
 
-from xdis.version_info import PYTHON_VERSION_TRIPLE, IS_PYPY
+from xdis.opcodes.opcode_3x import parse_fn_counts_30_35
+from xdis.version_info import IS_PYPY, PYTHON_VERSION_TRIPLE
+
 from xpython.byteop.byteop24 import Version_info
 from xpython.byteop.byteop32 import ByteOp32
 from xpython.byteop.byteop33 import ByteOp33
@@ -46,20 +48,22 @@ class ByteOp34(ByteOp33):
 
     # Python 3.3 docs describe this but seem to follow pre-3.3
     # conventions (which go back to Python 2.x days).
-    def MAKE_FUNCTION(self, argc: int):
+    def MAKE_FUNCTION(self, argc):
         """
-        Pushes a new function object on the stack. From bottom to top, the consumed stack must consist of:
+        Pushes a new function object on the stack. From bottom to top, the consumed
+        stack must consist of:
 
         * argc & 0xFF default argument objects in positional order
-        * (argc >> 8) & 0xFF pairs of name and default argument, with the name just below the object on the stack, for keyword-only parameters
+        * (argc >> 8) & 0xFF pairs of name and default argument, with the name just
+          below the object on the stack, for keyword-only parameters
         * (argc >> 16) & 0x7FFF parameter annotation objects
-        * a tuple listing the parameter names for the annotations (only if there are ony annotation objects)
+        * a tuple listing the parameter names for the annotations (only if there are
+          only annotation objects)
         * the code associated with the function (at TOS1)
         * the qualified name of the function (at TOS)
         """
 
-        rest, default_count = divmod(argc, 256)
-        annotate_count, kw_default_count = divmod(rest, 256)
+        default_count, kw_default_count, annotate_count = parse_fn_counts_30_35(argc)
 
         name = self.vm.pop()
         code = self.vm.pop()
@@ -67,8 +71,14 @@ class ByteOp34(ByteOp33):
             annotate_names = self.vm.pop()
             # annotate count includes +1 for the above names
             annotate_objects = self.vm.popn(annotate_count - 1)
-            n = len(annotate_names)
-            assert n == len(annotate_objects)
+            n = len(annotate_objects)
+            # We can annotate a function return value, but
+            # that does is not a parameter object and not
+            # listed in annotate_objects
+            if annotate_names[-1] == "return":
+                assert n == len(annotate_names) - 1
+            else:
+                assert n == len(annotate_names)
             annotations = {annotate_names[i]: annotate_objects[i] for i in range(n)}
         else:
             annotations = {}
@@ -113,7 +123,7 @@ class ByteOp34(ByteOp33):
 
             native_fn = types.FunctionType(code, globs, name, tuple(defaults))
             native_fn.__kwdefaults__ = kwdefaults
-            native_fn.__annonations__ = annotations
+            native_fn.__annotations__ = annotations
             self.vm.fn2native[fn] = native_fn
 
         self.vm.push(fn)
