@@ -1,11 +1,13 @@
 import sys
 import py
 
-__all__ = ["ObjSpace", "OperationError", "W_Root"]
+from xpython.interpreter.error import OperationError
+
+__all__ = ["ObjSpace", "W_Root"]
 
 
 def get_printable_location(greenkey):
-    return "unpackiterable [%s]" % (greenkey.iterator_greenkey_printable(),)
+    return f"unpackiterable [{greenkey.iterator_greenkey_printable()}]"
 
 
 class W_Root(object):
@@ -93,7 +95,7 @@ class W_Root(object):
 
     def getrepr(self, space, info, moreinfo=""):
         addrstring = self.getaddrstring(space)
-        return space.newtext("<%s at 0x%s%s>" % (info, addrstring, moreinfo))
+        return space.newtext(f"<{info} at 0x{addrstring}{moreinfo}>")
 
     def getslotvalue(self, index):
         raise NotImplementedError
@@ -113,7 +115,7 @@ class W_Root(object):
             space.w_TypeError, "'%s' object expected, got '%T' instead", classname, self
         )
 
-    # used by _weakref implemenation
+    # used by _weakref implemtenation
 
     def getweakref(self):
         return None
@@ -392,7 +394,7 @@ def wrappable_class_name(Class):
     try:
         return Class.typedef.name
     except AttributeError:
-        return "internal subclass of %s" % (Class.__name__,)
+        return f"internal subclass of {Class.__name__}"
 
 
 class CannotHaveLock(Exception):
@@ -497,10 +499,10 @@ class ObjSpace(object):
     def setbuiltinmodule(self, pkgname):
         """load a lazy pypy/module and put it into sys.modules"""
         if "." in pkgname:
-            fullname = "%s.moduledef" % (pkgname,)
+            fullname = f"{pkgname}.moduledef"
             importname = pkgname.rsplit(".", 1)[1]
         else:
-            fullname = "pypy.module.%s.moduledef" % pkgname
+            fullname = f"pypy.module.{pkgname}.moduledef"
             importname = pkgname
 
         Module = __import__(fullname, None, None, ["Module"]).Module
@@ -1325,8 +1327,8 @@ class ObjSpace(object):
             """(foo, bar): return 'baz'"""
         '''
         source = source.lstrip()
-        assert source.startswith("("), "incorrect header in:\n%s" % (source,)
-        source = py.code.Source("def anonymous%s\n" % source)
+        assert source.startswith("("), f"incorrect header in:\n{source}"
+        source = py.code.Source(f"def anonymous{source}\n")
         w_glob = self.newdict(module=True)
         self.exec_(str(source), w_glob, w_glob)
         return self.getitem(w_glob, self.newtext("anonymous"))
@@ -1638,7 +1640,7 @@ class ObjSpace(object):
         We should kill str_w completely and manually substitute it with
         text_w/bytes_w at all call sites.  It remains for now for tests only.
         """
-        XXX  # deprecated, leaving in place for clear errors
+        # deprecated, leaving in place for clear errors
         if self.isinstance_w(w_obj, self.w_unicode):
             # XXX lo text_w, but better to deprecate str_w than to fix this
             return w_obj.text_w(self)
@@ -1762,11 +1764,7 @@ class ObjSpace(object):
                 tp = self.type(w_obj).name
                 self.warn(
                     self.newtext(
-                        "path should be %s, not %s"
-                        % (
-                            allowed_types,
-                            tp,
-                        )
+                        f"path should be {allowed_types}, not {tp}"
                     ),
                     self.w_DeprecationWarning,
                 )
@@ -2141,7 +2139,7 @@ ObjSpace.ExceptionTable = [
 if sys.platform.startswith("win"):
     ObjSpace.ExceptionTable += ["WindowsError"]
 
-## Irregular part of the interface:
+# Irregular part of the interface:
 #
 #                                   wrap(x) -> w_x
 #                              str_w(w_str) -> str
