@@ -5,7 +5,9 @@ import linecache
 import types
 from copy import copy
 from sys import stderr
+
 from xdis import CO_GENERATOR, CO_ITERABLE_COROUTINE, iscode
+from xdis.cross_dis import findlinestarts
 from xdis.version_info import PYTHON3, PYTHON_VERSION_TRIPLE
 
 if PYTHON_VERSION_TRIPLE >= (3, 4):
@@ -16,10 +18,8 @@ else:
         pass
 
 
-import xpython.stdlib.inspect3 as inspect3
 import xpython.stdlib.inspect2 as inspect2
-
-import six
+import xpython.stdlib.inspect3 as inspect3
 
 PY2 = not PYTHON3
 
@@ -37,14 +37,14 @@ def make_cell(value):
 
 # It might be the case that this is more useful in Python 2.x
 # which doesn't seem to show traceback of interpreted code.
-# Python 3.x does this but it also shows junk at the end.
+# Python 3.x does this, but it also shows junk at the end.
 Traceback = collections.namedtuple("_Traceback", "tb_frame tb_lasti tb_lineno tb_next")
 try:
-    _Traceback.tb_frame.__doc__ = "frame object at this level"
-    _Traceback.tb_lasti.__doc__ = "index of last attempted instruction in bytecode"
-    _Traceback.tb_lineno.__doc__ = "current line number in Python source code"
-    _Traceback.tb_next.__doc__ = "next inner traceback object (called by this level)"
-except:
+    Traceback.tb_frame.__doc__ = "frame object at this level"
+    Traceback.tb_lasti.__doc__ = "index of last attempted instruction in bytecode"
+    Traceback.tb_lineno.__doc__ = "current line number in Python source code"
+    Traceback.tb_next.__doc__ = "next inner traceback object (called by this level)"
+except Exception:
     pass
 
 
@@ -54,7 +54,7 @@ COMPREHENSION_FN_NAMES = frozenset(
 )
 
 
-class Function(object):
+class Function:
     """Function(name, code, globals, argdefs, closure, vm,  kwdefaults={},
                 annotations={}, doc=None, qualname=None)
 
@@ -100,8 +100,8 @@ class Function(object):
         code,
         globs,
         argdefs,
-        closure=None,
-        vm=None,
+        closure,
+        vm,
         kwdefaults={},
         annotations={},
         doc=None,
@@ -111,7 +111,7 @@ class Function(object):
         self.version = vm.version
         self.__doc__ = doc
 
-        if not name is None and not isinstance(name, str):
+        if name is not None and not isinstance(name, str):
             raise TypeError(
                 "Function() argument 1 (name) must None or string, not %s" % type(name)
             )
@@ -163,10 +163,14 @@ class Function(object):
             assert annotations == {}
             assert kwdefaults == {}
 
-        # In Python 3.x is varous generators and list comprehensions have a .0 arg
+        # In Python 3.x is various generators and list comprehensions have a .0 arg
         # but inspect doesn't show that. In the various MAKE_FUNCTION routines,
         # we will detect this and store True in this field when appropriate.
-        if not argdefs and self.__name__.split(".")[-1] in COMPREHENSION_FN_NAMES:
+        if (
+            not argdefs
+            and hasattr(self, "__name")
+            and self.__name__.split(".")[-1] in COMPREHENSION_FN_NAMES
+        ):
             self.has_dot_zero = True
         else:
             self.has_dot_zero = False
@@ -187,16 +191,20 @@ class Function(object):
         # alternatives, but overall more of this needs to be done.
         #
         # The intent in providing native functions is for use in type
-        # testing, mostly. The functios should not be run, since that defeats our
+        # testing, mostly. The functions should not be run, since that defeats our
         # ability to trace functions.
-        kw = {"argdefs": self.func_defaults}
+        if hasattr(self, "func_defaults"):
+            kw = {"argdefs": self.func_defaults}
+        else:
+            kw = {}
+
         if closure:
             kw["closure"] = tuple(make_cell(0) for _ in closure)
 
         if not isinstance(code, types.CodeType) and hasattr(code, "to_native"):
             try:
                 code = code.to_native()
-            except:
+            except Exception:
                 pass
 
         if isinstance(code, types.CodeType):
@@ -209,14 +217,17 @@ class Function(object):
                     self._func.__kwdefaults__ = kwdefaults
                     self._func.__annotations__ = annotations
                     pass
-            except:
+            except Exception:
                 self._func = None
         else:
             # cross version interpreting... FIXME: fix this up
             self._func = None
 
     def __repr__(self):  # pragma: no cover
-        return "<Function %s at 0x%08x>" % (self.func_name, id(self))
+        if hasattr(self, "func_name"):
+            return "<Function %s at 0x%08x>" % (self.func_name, id(self))
+        elif hasattr(self, "_func"):
+            return str(self._func)
 
     def __get__(self, instance, owner):
         if instance is not None:
@@ -347,7 +358,7 @@ class Block(object):
 
     The equivalent of CPython's PyFrame_BlockSetup()
 
-    The are used in "try" and "with" statements; before Python 3.8 also in opcodes
+    They are used in "try" and "with" statements; before Python 3.8 also in opcodes
     associated with `for` (`SETUP_LOOP`) and `except` (`SETUP_EXCEPT`).
 
     From Python Include/frameobject.h:
@@ -392,6 +403,9 @@ class Frame(object):
         self.stack = []
         self.f_trace = None
 
+        # event args is used in tracing/debugging callback.
+        self.event_flags = None
+
         # brkpt is a mapping bytecode offset to the opcode value that was
         # smasshed by overwriting it with the pseudo opcode BRKPT.
         # After a breakpoint is serviced, this opcode needs to be run.
@@ -438,8 +452,8 @@ class Frame(object):
                     self.cells[var] = closure[i]
                 else:
                     # FIXME: this branch is probably wrong.
-                    # Also check all calls of Frame and make_frame() in vm to ensure we pass a function's
-                    # closure attribute.
+                    # Also check all calls of Frame and make_frame() in vm to ensure we
+                    # pass a function's closure attribute.
                     assert isinstance(f_back.cells[i], Cell), "f_back.cells[%d]: %r" % (
                         i,
                         f_back.cells[i],
@@ -452,11 +466,14 @@ class Frame(object):
         self.generator = None
         self.version = version
 
-        # These are sentinal or bogus values to start out.
+        # These are sentinel or bogus values to start out.
         # eval_frame will adjust inst_index.
         self.inst_index = -1
         self.fallthrough = False
         self.last_op = None
+
+        # Keep a cache of line starts for this frame
+        self.line_starts = list(findlinestarts(self.f_code))
         return
 
     def __repr__(self):  # pragma: no cover
@@ -467,30 +484,17 @@ class Frame(object):
             self.f_lasti,
         )
 
-    def line_number(self):
+    def line_number(self) -> int:
         """Get the current line number the frame is executing."""
         # We don't keep f_lineno up to date, so calculate it based on the
         # instruction address and the line number table.
-        lnotab = self.f_code.co_lnotab
-        byte_increments = six.iterbytes(lnotab[0::2])
-        line_increments = six.iterbytes(lnotab[1::2])
-
-        byte_num = 0
-        line_num = self.f_code.co_firstlineno
-
-        for byte_incr, line_incr in zip(byte_increments, line_increments):
-            if isinstance(byte_incr, str):
-                byte_incr = ord(byte_incr)
-
-            byte_num += byte_incr
-            if byte_num > self.f_lasti:
+        last_line_number = 0
+        for offset, line_number in self.line_starts:
+            if offset > self.f_lasti:
                 break
-            if isinstance(line_incr, str):
-                line_incr = ord(line_incr)
+            last_line_number = line_number
 
-            line_num += line_incr
-
-        return line_num
+        return last_line_number
 
 
 class Traceback(object):
@@ -565,7 +569,7 @@ class Generator(object):
 
 if __name__ == "__main__":
     frame = Frame(
-        traceback_from_frame.__code__, globals(), locals(), None, PYTHON_VERSION
+        traceback_from_frame.__code__, globals(), locals(), None, PYTHON_VERSION_TRIPLE
     )
     print(frame)
 
