@@ -2,11 +2,8 @@
 """Parts of Python 3.3+'s inspect class for older Pythons"""
 
 import itertools
-import types
 from collections import namedtuple, OrderedDict
 
-from xdis import PYTHON_VERSION
-
 # from _ParameterKind without the enum that doesn't work on older Pythons
 POSITIONAL_ONLY = 0
 POSITIONAL_OR_KEYWORD = 1
@@ -14,33 +11,41 @@ VAR_POSITIONAL = 2
 KEYWORD_ONLY = 3
 VAR_KEYWORD = 4
 
+
 def formatannotation(annotation, base_module=None):
-    if getattr(annotation, '__module__', None) == 'typing':
-        return repr(annotation).replace('typing.', '')
+    if getattr(annotation, "__module__", None) == "typing":
+        return repr(annotation).replace("typing.", "")
     if isinstance(annotation, type):
-        if annotation.__module__ in ('builtins', base_module):
+        if annotation.__module__ in ("builtins", base_module):
             return annotation.__qualname__
-        return annotation.__module__+'.'+annotation.__qualname__
+        return annotation.__module__ + "." + annotation.__qualname__
     return repr(annotation)
 
+
 class _void:
     """A private marker - used in Parameter & Signature."""
 
+
 class _empty:
     """Marker object for Signature.empty and Parameter.empty."""
+
     pass
 
+
 _PARAM_NAME_MAPPING = {
-    POSITIONAL_ONLY: 'positional-only',
-    POSITIONAL_OR_KEYWORD: 'positional or keyword',
-    VAR_POSITIONAL: 'variadic positional',
-    KEYWORD_ONLY: 'keyword-only',
-    VAR_KEYWORD: 'variadic keyword'
+    POSITIONAL_ONLY: "positional-only",
+    POSITIONAL_OR_KEYWORD: "positional or keyword",
+    VAR_POSITIONAL: "variadic positional",
+    KEYWORD_ONLY: "keyword-only",
+    VAR_KEYWORD: "variadic keyword",
 }
 _get_paramkind_descr = _PARAM_NAME_MAPPING.__getitem__
 
-FullArgSpec = namedtuple('FullArgSpec',
-    'args, varargs, varkw, defaults, kwonlyargs, kwonlydefaults, annotations')
+FullArgSpec = namedtuple(
+    "FullArgSpec",
+    "args, varargs, varkw, defaults, kwonlyargs, kwonlydefaults, annotations",
+)
+
 
 class Parameter:
     """Represents a parameter in a function signature.
@@ -64,7 +69,7 @@ class Parameter:
         `Parameter.KEYWORD_ONLY`, `Parameter.VAR_KEYWORD`.
     """
 
-    __slots__ = ('_name', '_kind', '_default', '_annotation')
+    __slots__ = ("_name", "_kind", "_default", "_annotation")
 
     empty = _empty
 
@@ -72,51 +77,52 @@ class Parameter:
         try:
             self._kind = kind
         except ValueError:
-            raise ValueError('value {%r} is not a valid Parameter.kind' % kind)
+            raise ValueError("value {%r} is not a valid Parameter.kind" % kind)
         if default is not _empty:
             if self._kind in (VAR_POSITIONAL, VAR_KEYWORD):
-                msg = '{} parameters cannot have default values'
+                msg = "{} parameters cannot have default values"
                 msg = msg.format(_get_paramkind_descr(self._kind))
                 raise ValueError(msg)
         self._default = default
         self._annotation = annotation
 
         if name is _empty:
-            raise ValueError('name is a required attribute for Parameter')
+            raise ValueError("name is a required attribute for Parameter")
 
         if not isinstance(name, str):
-            msg = 'name must be a str, not a {}'.format(type(name).__name__)
+            msg = "name must be a str, not a {}".format(type(name).__name__)
             raise TypeError(msg)
 
-        if name[0] == '.' and name[1:].isdigit():
+        if name[0] == "." and name[1:].isdigit():
             # These are implicit arguments generated by comprehensions. In
             # order to provide a friendlier interface to users, we recast
             # their name as "implicitN" and treat them as positional-only.
             # See issue 19611.
             if self._kind != POSITIONAL_OR_KEYWORD:
                 msg = (
-                    'implicit arguments must be passed as '
-                    'positional or keyword arguments, not {}'
+                    "implicit arguments must be passed as "
+                    "positional or keyword arguments, not {}"
                 )
                 msg = msg.format(_get_paramkind_descr(self._kind))
                 raise ValueError(msg)
             self._kind = POSITIONAL_ONLY
-            name = 'implicit{}'.format(name[1:])
+            name = "implicit{}".format(name[1:])
 
         if not name.isidentifier():
-            raise ValueError('{!r} is not a valid parameter name'.format(name))
+            raise ValueError("{!r} is not a valid parameter name".format(name))
 
         self._name = name
 
     def __reduce__(self):
-        return (type(self),
-                (self._name, self._kind),
-                {'_default': self._default,
-                 '_annotation': self._annotation})
+        return (
+            type(self),
+            (self._name, self._kind),
+            {"_default": self._default, "_annotation": self._annotation},
+        )
 
     def __setstate__(self, state):
-        self._default = state['_default']
-        self._annotation = state['_annotation']
+        self._default = state["_default"]
+        self._annotation = state["_annotation"]
 
     @property
     def name(self):
@@ -134,8 +140,7 @@ class Parameter:
     def kind(self):
         return self._kind
 
-    def replace(self, name=_void, kind=_void,
-                annotation=_void, default=_void):
+    def replace(self, name=_void, kind=_void, annotation=_void, default=_void):
         """Creates a customized copy of the Parameter."""
 
         if name is _void:
@@ -158,19 +163,18 @@ class Parameter:
 
         # Add annotation and default value
         if self._annotation is not _empty:
-            formatted = '{}: {}'.format(formatted,
-                                       formatannotation(self._annotation))
+            formatted = "{}: {}".format(formatted, formatannotation(self._annotation))
 
         if self._default is not _empty:
             if self._annotation is not _empty:
-                formatted = '{} = {}'.format(formatted, repr(self._default))
+                formatted = "{} = {}".format(formatted, repr(self._default))
             else:
-                formatted = '{}={}'.format(formatted, repr(self._default))
+                formatted = "{}={}".format(formatted, repr(self._default))
 
         if kind == VAR_POSITIONAL:
-            formatted = '*' + formatted
+            formatted = "*" + formatted
         elif kind == VAR_KEYWORD:
-            formatted = '**' + formatted
+            formatted = "**" + formatted
 
         return formatted
 
@@ -185,10 +189,12 @@ class Parameter:
             return True
         if not isinstance(other, Parameter):
             return NotImplemented
-        return (self._name == other._name and
-                self._kind == other._kind and
-                self._default == other._default and
-                self._annotation == other._annotation)
+        return (
+            self._name == other._name
+            and self._kind == other._kind
+            and self._default == other._default
+            and self._annotation == other._annotation
+        )
 
 
 class BoundArguments:
@@ -208,7 +214,7 @@ class BoundArguments:
         Dict of keyword arguments values.
     """
 
-    __slots__ = ('arguments', '_signature', '__weakref__')
+    __slots__ = ("arguments", "_signature", "__weakref__")
 
     def __init__(self, signature, arguments):
         self.arguments = arguments
@@ -304,21 +310,20 @@ class BoundArguments:
             return True
         if not isinstance(other, BoundArguments):
             return NotImplemented
-        return (self.signature == other.signature and
-                self.arguments == other.arguments)
+        return self.signature == other.signature and self.arguments == other.arguments
 
     def __setstate__(self, state):
-        self._signature = state['_signature']
-        self.arguments = state['arguments']
+        self._signature = state["_signature"]
+        self.arguments = state["arguments"]
 
     def __getstate__(self):
-        return {'_signature': self._signature, 'arguments': self.arguments}
+        return {"_signature": self._signature, "arguments": self.arguments}
 
     def __repr__(self):
         args = []
         for arg, value in self.arguments.items():
-            args.append('{}={!r}'.format(arg, value))
-        return '<{} ({})>'.format(self.__class__.__name__, ', '.join(args))
+            args.append("{}={!r}".format(arg, value))
+        return "<{} ({})>".format(self.__class__.__name__, ", ".join(args))
 
 
 class Signature:
@@ -344,15 +349,16 @@ class Signature:
         to parameters (simulating 'functools.partial' behavior.)
     """
 
-    __slots__ = ('_return_annotation', '_parameters')
+    __slots__ = ("_return_annotation", "_parameters")
 
     _parameter_cls = Parameter
     _bound_arguments_cls = BoundArguments
 
     empty = _empty
 
-    def __init__(self, parameters=None, return_annotation=_empty,
-                 __validate_parameters__=True):
+    def __init__(
+        self, parameters=None, return_annotation=_empty, __validate_parameters__=True
+    ):
         """Constructs Signature from the given list of Parameter
         objects and 'return_annotation'.  All arguments are optional.
         """
@@ -371,11 +377,11 @@ class Signature:
 
                     if kind < top_kind:
                         msg = (
-                            'wrong parameter order: {} parameter before {} '
-                            'parameter'
+                            "wrong parameter order: {} parameter before {} " "parameter"
+                        )
+                        msg = msg.format(
+                            _get_paramkind_descr(top_kind), _get_paramkind_descr(kind)
                         )
-                        msg = msg.format(_get_paramkind_descr(top_kind),
-                                         _get_paramkind_descr(kind))
                         raise ValueError(msg)
                     elif kind > top_kind:
                         kind_defaults = False
@@ -387,21 +393,19 @@ class Signature:
                                 # No default for this parameter, but the
                                 # previous parameter of the same kind had
                                 # a default
-                                msg = 'non-default argument follows default ' \
-                                      'argument'
+                                msg = "non-default argument follows default " "argument"
                                 raise ValueError(msg)
                         else:
                             # There is a default for this parameter.
                             kind_defaults = True
 
                     if name in params:
-                        msg = 'duplicate parameter name: {!r}'.format(name)
+                        msg = "duplicate parameter name: {!r}".format(name)
                         raise ValueError(msg)
 
                     params[name] = param
             else:
-                params = OrderedDict(((param.name, param)
-                                                for param in parameters))
+                params = OrderedDict(((param.name, param) for param in parameters))
 
         # types.MappingProxyType (which is "dict" isn't around in older Pythons
         # and dict(params) doesn't work.
@@ -462,15 +466,18 @@ class Signature:
         if return_annotation is _void:
             return_annotation = self._return_annotation
 
-        return type(self)(parameters,
-                          return_annotation=return_annotation)
+        return type(self)(parameters, return_annotation=return_annotation)
 
     def _hash_basis(self):
-        params = tuple(param for param in self.parameters.values()
-                             if param.kind != KEYWORD_ONLY)
+        params = tuple(
+            param for param in self.parameters.values() if param.kind != KEYWORD_ONLY
+        )
 
-        kwo_params = {param.name: param for param in self.parameters.values()
-                                        if param.kind == KEYWORD_ONLY}
+        kwo_params = {
+            param.name: param
+            for param in self.parameters.values()
+            if param.kind == KEYWORD_ONLY
+        }
 
         return params, kwo_params, self.return_annotation
 
@@ -515,14 +522,15 @@ class Signature:
                         break
                     elif param.name in kwargs:
                         if param.kind == POSITIONAL_ONLY:
-                            msg = '{arg!r} parameter is positional only, ' \
-                                  'but was passed as a keyword'
+                            msg = (
+                                "{arg!r} parameter is positional only, "
+                                "but was passed as a keyword"
+                            )
                             msg = msg.format(arg=param.name)
                             raise TypeError(msg)
                         parameters_ex = (param,)
                         break
-                    elif (param.kind == VAR_KEYWORD or
-                                                param.default is not _empty):
+                    elif param.kind == VAR_KEYWORD or param.default is not _empty:
                         # That's fine too - we have a default value for this
                         # parameter.  So, lets start parsing `kwargs`, starting
                         # with the current parameter
@@ -535,7 +543,7 @@ class Signature:
                             parameters_ex = (param,)
                             break
                         else:
-                            msg = 'missing a required argument: {arg!r}'
+                            msg = "missing a required argument: {arg!r}"
                             msg = msg.format(arg=param.name)
                             raise TypeError(msg)
             else:
@@ -543,13 +551,12 @@ class Signature:
                 try:
                     param = next(parameters)
                 except StopIteration:
-                    raise TypeError('too many positional arguments')
+                    raise TypeError("too many positional arguments")
                 else:
                     if param.kind in (VAR_KEYWORD, KEYWORD_ONLY):
                         # Looks like we have no parameter for this positional
                         # argument
-                        raise TypeError(
-                            'too many positional arguments')
+                        raise TypeError("too many positional arguments")
 
                     if param.kind == VAR_POSITIONAL:
                         # We have an '*args'-like argument, let's fill it with
@@ -562,8 +569,10 @@ class Signature:
 
                     if param.name in kwargs:
                         raise TypeError(
-                            'multiple values for argument {arg!r}'.format(
-                                arg=param.name))
+                            "multiple values for argument {arg!r}".format(
+                                arg=param.name
+                            )
+                        )
 
                     arguments[param.name] = arg_val
 
@@ -590,19 +599,24 @@ class Signature:
                 # if it has a default value, or it is an '*args'-like
                 # parameter, left alone by the processing of positional
                 # arguments.
-                if (not partial and param.kind != VAR_POSITIONAL and
-                                                    param.default is _empty):
-                    raise TypeError('missing a required argument: {arg!r}'. \
-                                    format(arg=param_name))
+                if (
+                    not partial
+                    and param.kind != VAR_POSITIONAL
+                    and param.default is _empty
+                ):
+                    raise TypeError(
+                        "missing a required argument: {arg!r}".format(arg=param_name)
+                    )
 
             else:
                 if param.kind == POSITIONAL_ONLY:
                     # This should never happen in case of a properly built
                     # Signature object (but let's have this check here
                     # to ensure correct behaviour just in case)
-                    raise TypeError('{arg!r} parameter is positional only, '
-                                    'but was passed as a keyword'. \
-                                    format(arg=param.name))
+                    raise TypeError(
+                        "{arg!r} parameter is positional only, "
+                        "but was passed as a keyword".format(arg=param.name)
+                    )
 
                 arguments[param_name] = arg_val
 
@@ -612,8 +626,10 @@ class Signature:
                 arguments[kwargs_param.name] = kwargs
             else:
                 raise TypeError(
-                    'got an unexpected keyword argument {arg!r}'.format(
-                        arg=next(iter(kwargs))))
+                    "got an unexpected keyword argument {arg!r}".format(
+                        arg=next(iter(kwargs))
+                    )
+                )
 
         return self._bound_arguments_cls(self, arguments)
 
@@ -632,15 +648,17 @@ class Signature:
         return args[0]._bind(args[1:], kwargs, partial=True)
 
     def __reduce__(self):
-        return (type(self),
-                (tuple(self._parameters.values()),),
-                {'_return_annotation': self._return_annotation})
+        return (
+            type(self),
+            (tuple(self._parameters.values()),),
+            {"_return_annotation": self._return_annotation},
+        )
 
     def __setstate__(self, state):
-        self._return_annotation = state['_return_annotation']
+        self._return_annotation = state["_return_annotation"]
 
     def __repr__(self):
-        return '<{} {}>'.format(self.__class__.__name__, self)
+        return "<{} {}>".format(self.__class__.__name__, self)
 
     def __str__(self):
         result = []
@@ -656,7 +674,7 @@ class Signature:
             elif render_pos_only_separator:
                 # It's not a positional-only parameter, and the flag
                 # is set to 'True' (there were pos-only params before.)
-                result.append('/')
+                result.append("/")
                 render_pos_only_separator = False
 
             if kind == VAR_POSITIONAL:
@@ -667,7 +685,7 @@ class Signature:
                 # We have a keyword-only parameter to render and we haven't
                 # rendered an '*args'-like parameter before, so add a '*'
                 # separator to the parameters list ("foo(arg1, *, arg2)" case)
-                result.append('*')
+                result.append("*")
                 # This condition should be only triggered once, so
                 # reset the flag
                 render_kw_only_separator = False
@@ -677,12 +695,12 @@ class Signature:
         if render_pos_only_separator:
             # There were only positional-only parameters, hence the
             # flag was not reset to 'False'
-            result.append('/')
+            result.append("/")
 
-        rendered = '({})'.format(', '.join(result))
+        rendered = "({})".format(", ".join(result))
 
         if self.return_annotation is not _empty:
             anno = formatannotation(self.return_annotation)
-            rendered += ' -> {}'.format(anno)
+            rendered += " -> {}".format(anno)
 
         return rendered
