@@ -1,3 +1,6 @@
+"""
+List of bytecode and Python versions that xpython has support for
+"""
 # The below has extra commas for POSIX shell's taste, but that's okay
 SUPPORTED_PYTHON = (
     (2, 7),
@@ -10,10 +13,11 @@ SUPPORTED_PYTHON = (
     (3, 8),
     (3, 9),
     (3, 10),
+    (3, 11),
+    (3, 12),
 )
 
-# PYPY 3.7 and 3.8 aren't ready yet
-SUPPORTED_PYPY = ((2, 7), (3, 2), (3, 5), (3, 6), (3, 7), (3, 8))  # noqa
+SUPPORTED_PYPY = ((2, 7), (3, 2), (3, 5), (3, 6), (3, 7), (3, 8), (3, 9))  # noqa
 
 SUPPORTED_BYTECODE = (
     (2, 4),
@@ -29,4 +33,6 @@ SUPPORTED_BYTECODE = (
     (3, 8),
     (3, 9),
     (3, 10),
+    (3, 11),
+    (3, 12),
 )
