@@ -7,26 +7,23 @@ import sys
 
 import six
 from six.moves import reprlib
-
-from xdis import (
-    code2num,
-    CO_NEWLOCALS,
-    PYTHON3,
-    PYTHON_VERSION_TRIPLE,
-    IS_PYPY,
-    op_has_argument,
-    next_offset,
-)
+from xdis import (CO_NEWLOCALS, IS_PYPY, PYTHON3, PYTHON_VERSION_TRIPLE,
+                  code2num, next_offset, op_has_argument)
+from xdis.cross_types import UnicodeForPython3
 from xdis.op_imports import get_opcode_module
+from xdis.opcodes.opcode_311 import _nb_ops
 
-from xpython.pyobj import Frame, Block, Traceback, traceback_from_frame
 from xpython.byteop import get_byteop
+from xpython.pyobj import Block, Frame, Traceback, traceback_from_frame
 
 PY2 = not PYTHON3
 log = logging.getLogger(__name__)
 
 if PYTHON3:
-    byteint = lambda b: b
+
+    def byteint(b):
+        return b
+
 else:
     byteint = ord
 
@@ -97,7 +94,7 @@ def format_instruction(
     vm=None,
 ):
     """Formats an instruction. What's a little different here is that in
-    contast to Python's `dis`, or a colorized version of that, used in
+    contrast to Python's `dis`, or a colorized version of that, used in
     `trepan3k` we may have access to the frame eval stack and therefore
     can show operands in a nicer way.
 
@@ -113,7 +110,7 @@ def format_instruction(
         stack_args = ""
 
     if hasattr(opc, "opcode_arg_fmt") and bytecode_name in opc.opcode_arg_fmt:
-        argrepr = f"""[{opc.opcode_arg_fmt[bytecode_name](int_arg)}] {int_arg}"""
+        argrepr = opc.opcode_arg_fmt[bytecode_name](int_arg)
     elif int_arg is None:
         argrepr = ""
     elif byte_code in opc.COMPARE_OPS:
@@ -154,6 +151,8 @@ class PyVM(object):
         self.is_pypy = is_pypy
         self.format_instruction = format_instruction_func
 
+        self.null_ops = []
+
         # FIXME: until we figure out how to fix up test/vmtest.el
         # This changes how we report a VMRuntime error.
         self.vmtest_testing = vmtest_testing
@@ -177,9 +176,8 @@ class PyVM(object):
 
         variant = "pypy" if is_pypy else None
 
-        # FIXME: HACK alert - until the next release of xpython > 6.0.3
-        if is_pypy and python_version == (3, 8, 0):
-            python_version = (3, 8, 12)
+        if is_pypy:
+            python_version = tuple(python_version[:2])
 
         self.opc = get_opcode_module(python_version, variant)
         self.byteop = get_byteop(self, python_version, is_pypy)
@@ -199,7 +197,7 @@ class PyVM(object):
             raise PyVMError("Peek value must be greater than 0")
         try:
             return self.frame.stack[-n]
-        except:
+        except Exception:
             return 0
 
     def pop(self, i=0):
@@ -228,6 +226,10 @@ class PyVM(object):
         """Push values onto the value stack."""
         self.frame.stack.extend(vals)
 
+    def set(self, i: int, value):
+        """Set a value at stack position i."""
+        self.frame.stack[-i] = value
+
     def top(self):
         """Return the value at the top of the stack, with no changes."""
         return self.frame.stack[-1]
@@ -261,6 +263,14 @@ class PyVM(object):
         self.frame.f_lasti = jump
         self.frame.fallthrough = False
 
+    def jump_relative(self, delta: int):
+        """Adjust the bytecode pointer by `deleta`, so it will execute next,
+        However we subtract one from the offset, because fetching the
+        next instruction adds one before fetching.
+        """
+        self.frame.f_lasti += delta
+        self.frame.fallthrough = False
+
     def make_frame(
         self, code, callargs={}, f_globals=None, f_locals=None, closure=None
     ):
@@ -461,6 +471,8 @@ class PyVM(object):
 
                 if byte_code in self.opc.CONST_OPS:
                     arg = f_code.co_consts[int_arg]
+                    if isinstance(arg, UnicodeForPython3):
+                        arg = str(arg)
                 elif byte_code in self.opc.FREE_OPS:
                     if int_arg < len(f_code.co_cellvars):
                         arg = f_code.co_cellvars[int_arg]
@@ -469,6 +481,8 @@ class PyVM(object):
                         arg = f_code.co_freevars[var_idx]
                 elif byte_code in self.opc.NAME_OPS:
                     arg = f_code.co_names[int_arg]
+                    if isinstance(arg, UnicodeForPython3):
+                        arg = str(arg)
                 elif byte_code in self.opc.JREL_OPS:
                     # Many relative jumps are conditional,
                     # so setting f.fallthrough is wrong.
@@ -484,6 +498,8 @@ class PyVM(object):
                     arg = int_arg
                 elif byte_code in self.opc.LOCAL_OPS:
                     arg = f_code.co_varnames[int_arg]
+                    if isinstance(arg, UnicodeForPython3):
+                        arg = str(arg)
                 else:
                     arg = int_arg
                 arguments = [arg]
@@ -523,7 +539,11 @@ class PyVM(object):
             if bytecode_name.startswith("UNARY_"):
                 byteop.unaryOperator(bytecode_name[6:])
             elif bytecode_name.startswith("BINARY_"):
-                byteop.binaryOperator(bytecode_name[7:])
+                if self.version < (3, 11):
+                    byteop.binaryOperator(bytecode_name[7:])
+                else:
+                    byteop.binaryOperator(_nb_ops[int_arg][0][3:])
+
             elif bytecode_name.startswith("INPLACE_"):
                 byteop.inplaceOperator(bytecode_name[8:])
             elif "SLICE+" in bytecode_name:
@@ -532,6 +552,8 @@ class PyVM(object):
                 # dispatch
                 if hasattr(byteop, bytecode_name):
                     bytecode_fn = getattr(byteop, bytecode_name, None)
+                else:
+                    bytecode_fn = None
                 if not bytecode_fn:  # pragma: no cover
                     raise PyVMError(
                         "Unknown bytecode type: %s\n\t%s"
@@ -551,7 +573,7 @@ class PyVM(object):
                     )
                 why = bytecode_fn(*arguments)
 
-        except:
+        except Exception:
             # Deal with exceptions encountered while executing the op.
             self.last_exception = sys.exc_info()
 
@@ -673,7 +695,6 @@ class PyVM(object):
         self.push_frame(frame)
         offset = 0
         while True:
-
             (
                 bytecode_name,
                 byte_code,
@@ -745,13 +766,14 @@ class PyVM(object):
             else:
                 raise PyVMError("Borked exception recording")
             # if self.exception and .... ?
-            # log.error("Haven't finished traceback handling, nulling traceback information for now")
+            # log.error("Haven't finished traceback handling, nulling traceback "
+            #            "information for now")
             # six.reraise(self.last_exception[0], None)
 
         self.in_exception_processing = False
         return self.return_value
 
-    ## Operators
+    # Operators
 
     def sliceOperator(self, op):
         start = 0
@@ -764,19 +786,18 @@ class PyVM(object):
         elif count == 3:
             end = self.pop()
             start = self.pop()
-        l = self.pop()
+        slice_len = self.pop()
         if end is None:
-            end = len(l)
+            end = len(slice_len)
         if op.startswith("STORE_"):
-            l[start:end] = self.pop()
+            slice_len[start:end] = self.pop()
         elif op.startswith("DELETE_"):
-            del l[start:end]
+            del slice_len[start:end]
         else:
-            self.push(l[start:end])
+            self.push(slice_len[start:end])
 
 
 if __name__ == "__main__":
-
     # Simplest of tests
     def five():
         return 5
